#
#  MainApp.py
#
#  Created by jchrist on 1/8/08

#import modules required by application
import objc
import Foundation
import AppKit
from appscript import *
import os
import time 

from Logger import *

from PFRAppController import *
from PFRMenus import *
from PFROptionDialog import *

# Import backrow
objc.loadBundle("BackRow", globals(), bundle_path=objc.pathForFramework("/System/Library/PrivateFrameworks/BackRow.framework" ))

#
# modify this to set a unique key for your application, should match what's in setup.py
#
def GetAppKey():
    return u"com.apple.frontrow.appliance." + "pyetv"

#is EyeTV playing a recording?
def NotPlaying():
    return not app("EyeTV").playing.get()

#is the fullscreen menu up in EyeTV
def MenuGone():
    log("In MenuGone()")
    return not app("EyeTV").full_screen_menu.get()

def InstallExitHandler(controller):
    log("installing exit handler")
    time.sleep(1.0) # allow time for fs menu to stabilize before installing exit handler
    controller.setExitConditionTest(MenuGone)

def PlayFullScreenTimeout(controller):
    app('EyeTV').enter_full_screen()

# once EyeTV has started, enter full screen mode
def EnterFullScreenTimeout(controller, install_exit_handler=True):
    log("Got etv started callback")
    etvGUI = app('System Events').processes['EyeTV']
    etvGUI.frontmost(True)
    mref = etvGUI.menu_bars[1].menus
    mref['View'].menu_items['Open Menu'].click()
    app('EyeTV').enter_full_screen()

    if (install_exit_handler):
        InstallExitHandler(controller)

# once EyeTV has started, enter full screen mode and show the guide
def ShowProgramGuideTimeout(controller):
    EnterFullScreenTimeout(controller,False)
    app("System Events").keystroke("g",using=k.command_down)
    InstallExitHandler(controller)

# menu handler for showing the ETV program guide
def ShowGuide_MenuHandler(controller,arg):
    log("in ShowGuide_MenuHandler")
    newCon=PFRAppController.alloc().initWithAppPath_AppName_("/Applications/EyeTV.app","EyeTV")
    newCon.setUserTimeoutFunc(ShowProgramGuideTimeout,8) # 8 tics
    return controller.stack().pushController_(newCon)

# menu handler for showing the ETV fs menu
def StartETV_MenuHandler(controller,arg):
    log("in StartETV_MenuHandler")
    newCon=PFRAppController.alloc().initWithAppPath_AppName_("/Applications/EyeTV.app","EyeTV")
    newCon.setUserTimeoutFunc(EnterFullScreenTimeout,8) # 8 tics
    return controller.stack().pushController_(newCon)

COMSKIP_IS_ON="ON"
def HasSkipFile(arg):
    return "Not Found"

def PlayRecordingOptions_Handler(controller, idx, arg):
    log("PlayRecordingOptions_Handler")
    alert = BRAlertController.alertOfType_titled_primaryText_secondaryText_( 0, "Option response:", "Option #%s" % str(idx), "Userdata: %s" % arg)
    controller.stack().pushController_(alert)

    if idx==1:
        arg.playback_position.set(0)
        idx=0 # make fall through to playback mode

    if idx==0:
        newCon=PFRAppController.alloc().initWithAppPath_AppName_("/Applications/EyeTV.app","EyeTV")
        newCon.setUserTimeoutFunc(PlayFullScreenTimeout,8) # 8 tics
        app("EyeTV").play(arg)
        return controller.stack().pushController_(newCon)
    if idx==2:
        log("deletion request");
    if idx==3:
        log("comskip toggle request");
    if idx==4:
        log("skipfile scan request");

    # if we return true, we'll pop the controller and back up past the option dialog
    return False

# menu handler for showing the ETV fs menu
def PlayRecording_MenuHandler(controller,arg):
    log("in PlayRecording_MenuHandler")
    pos=arg.playback_position.get()
    end=arg.duration.get();
    smin=pos/60 # integer division
    ssec = pos - smin*60
    emin=end/60 # integer division
    esec = pos - emin*60

    options=[ "Play / Resume (%d:%02d of %d:%02d) " % (smin,ssec,emin,esec),
              "Restart",
              "Delete",
              "Comskip [%s]" % COMSKIP_IS_ON,
              "SkipFile [%s]" % HasSkipFile(arg) ]  

    dlg=PFROptionDialog.alloc().initWithTitle_Items_Handler_UserData_("Show menu",options,PlayRecordingOptions_Handler,arg)
    return controller.stack().pushController_(dlg)

# menu handler for showing the ETV fs menu
def TestText_MenuHandler(controller,arg):
    log("in Test_MenuHandler")
    newCon=BRTextEntryController.alloc().init()
    newCon.setTitle_("Type here:")
    newCon.setPromptText_("Prompt:")
    newCon.setShowUserEnteredText_(True)
    newCon.setInitialTextEntryText_("Hi there!")
    return controller.stack().pushController_(newCon)


def GetRecordingsDict():
    series_dict={}
    rec=app("EyeTV").recordings.get()
    for r in rec:
        title=r.title.get()
        series_dict[title]=[]

    for r in rec:
        title=r.title.get()
        series_dict[title].append(r)

    return series_dict


def MakeRecordingsMenu():
    root=PFRMenu("Recordings")
    series=GetRecordingsDict()

    k=series.keys()
    k.sort()
    for s in k:
        subtitle=" [%d]" % len(series[s])
        submenu=PFRMenu(s + subtitle,[])
        root.AddItem(submenu)
        for ep in series[s]:
            epstr=ep.episode.get() + " " +  str(ep.start_time.get())
            item=PFRMenuItem(epstr,PlayRecording_MenuHandler,ep)
            submenu.AddItem(item)
    return root



#
# return the type of controller you want here
#
def GetAppController():
    log("**************************************************")
    log("Appliance controller starting")

    #ETV=ETVController()

    recordings=MakeRecordingsMenu()
    log("Recordings menu has %d" % len(recordings.items))

    menu=PFRMenu("PyeTV",
                [
            recordings,
            PFRMenuItem("Enter EyeTV",   StartETV_MenuHandler,  "Enter EyeTV"),
            PFRMenuItem("Program guide", ShowGuide_MenuHandler, "Program Guide")
#            ,PFRMenuItem("Comskip off",   ShowGuide_MenuHandler, "Comskip on")
#            ,PFRMenuItem("Comskip off",   ShowGuide_MenuHandler, "Comskip off")
            ,PFRMenuItem("Option dialog",   testPFROptionDialogTest, "Text test")
#            ,PFRMenuItem("Text test",   TestText_MenuHandler, "Text test")
            ])
    ac=PFRMenuController.alloc().initWithMenu_(menu)
    log("Enabling logger")
    a=EnableObjcLogger()
    return ac
